from typing import Optional, Sequence

from kubragen import KubraGen
from kubragen.builder import Builder
from kubragen.configfile import ConfigFileRenderMulti, ConfigFileRender_Yaml, ConfigFileRender_RawStr
from kubragen.data import ValueData
from kubragen.exception import InvalidParamError, InvalidNameError
from kubragen.helper import LiteralStr, QuotedStr
from kubragen.object import ObjectItem, Object
from kubragen.types import TBuild, TBuildItem

from .configfile import PromtailConfigFile
from .option import PromtailOptions


class PromtailBuilder(Builder):
    """
    Promtail builder.

    Based on `Install Loki with Helm <https://grafana.com/docs/loki/latest/installation/helm/>`_.

    .. list-table::
        :header-rows: 1

        * - build
          - description
        * - BUILD_ACCESSCONTROL
          - creates service account, roles, and roles bindings
        * - BUILD_CONFIG
          - creates ConfigMap and Secret
        * - BUILD_SERVICE
          - creates deployments and services

    .. list-table::
        :header-rows: 1

        * - build item
          - description
        * - BUILDITEM_CONFIG
          - ConfigMap
        * - BUILDITEM_SERVICE_ACCOUNT
          - ServiceAccount
        * - BUILDITEM_CLUSTER_ROLE
          - ClusterRole
        * - BUILDITEM_CLUSTER_ROLE_BINDING
          - ClusterRoleBinding
        * - BUILDITEM_DAEMONSET
          - Daemonset

    .. list-table::
        :header-rows: 1

        * - object name
          - description
          - default value
        * - config
          - ConfigMap
          - ```<basename>-config```
        * - service-account
          - ServiceAccount
          - ```<basename>```
        * - cluster-role
          - cluster role
          - ```<basename>```
        * - cluster-role-binding
          - cluster role binding
          - ```<basename>```
        * - daemonset
          - DaemonSet
          - ```<basename>```
        * - pod-label-app
          - label *app* to be used by selection
          - ```<basename>```
    """
    options: PromtailOptions
    configfile: Optional[str]
    _namespace: str

    SOURCE_NAME = 'kg_promtail'

    BUILD_ACCESSCONTROL: TBuild = 'accesscontrol'
    BUILD_CONFIG: TBuild = 'config'
    BUILD_SERVICE: TBuild = 'service'

    BUILDITEM_CONFIG: TBuildItem = 'config'
    BUILDITEM_SERVICE_ACCOUNT: TBuildItem = 'service-account'
    BUILDITEM_CLUSTER_ROLE: TBuildItem = 'cluster-role'
    BUILDITEM_CLUSTER_ROLE_BINDING: TBuildItem = 'cluster-role-binding'
    BUILDITEM_DAEMONSET: TBuildItem = 'daemonset'

    def __init__(self, kubragen: KubraGen, options: Optional[PromtailOptions] = None):
        super().__init__(kubragen)
        if options is None:
            options = PromtailOptions()
        self.options = options
        self.configfile = None

        self._namespace = self.option_get('namespace')

        if self.option_get('config.authorization.serviceaccount_create') is not False:
            serviceaccount_name = self.basename()
        else:
            serviceaccount_name = self.option_get('config.authorization.serviceaccount_use')
            if serviceaccount_name == '':
                serviceaccount_name = None

        if self.option_get('config.authorization.roles_bind') is not False:
            if serviceaccount_name is None:
                raise InvalidParamError('To bind roles a service account is required')

        self.object_names_update({
            'config': self.basename('-config'),
            'service-account': serviceaccount_name,
            'cluster-role': self.basename(),
            'cluster-role-binding': self.basename(),
            'daemonset': self.basename(),
            'pod-label-app': self.basename(),
        })

    def option_get(self, name: str):
        return self.kubragen.option_root_get(self.options, name)

    def basename(self, suffix: str = ''):
        return '{}{}'.format(self.option_get('basename'), suffix)

    def namespace(self):
        return self._namespace

    def build_names(self) -> Sequence[TBuild]:
        return [self.BUILD_ACCESSCONTROL, self.BUILD_CONFIG, self.BUILD_SERVICE]

    def build_names_required(self) -> Sequence[TBuild]:
        ret = [self.BUILD_CONFIG, self.BUILD_SERVICE]
        if self.option_get('config.authorization.serviceaccount_create') is not False or \
                self.option_get('config.authorization.roles_create') is not False:
            ret.append(self.BUILD_ACCESSCONTROL)
        return ret

    def builditem_names(self) -> Sequence[TBuildItem]:
        return [
            self.BUILDITEM_CONFIG,
            self.BUILDITEM_SERVICE_ACCOUNT,
            self.BUILDITEM_CLUSTER_ROLE,
            self.BUILDITEM_CLUSTER_ROLE_BINDING,
            self.BUILDITEM_DAEMONSET,
        ]

    def internal_build(self, buildname: TBuild) -> Sequence[ObjectItem]:
        if buildname == self.BUILD_ACCESSCONTROL:
            return self.internal_build_accesscontrol()
        elif buildname == self.BUILD_CONFIG:
            return self.internal_build_config()
        elif buildname == self.BUILD_SERVICE:
            return self.internal_build_service()
        else:
            raise InvalidNameError('Invalid build name: "{}"'.format(buildname))

    def internal_build_accesscontrol(self) -> Sequence[ObjectItem]:
        ret = []

        if self.option_get('config.authorization.serviceaccount_create') is not False:
            ret.extend([
                Object({
                    'apiVersion': 'v1',
                    'kind': 'ServiceAccount',
                    'metadata': {
                        'name': self.object_name('service-account'),
                        'namespace': self.namespace(),
                    }
                }, name=self.BUILDITEM_SERVICE_ACCOUNT, source=self.SOURCE_NAME, instance=self.basename()),
            ])

        if self.option_get('config.authorization.roles_create') is not False:
            ret.extend([
                Object({
                    'kind': 'ClusterRole',
                    'apiVersion': 'rbac.authorization.k8s.io/v1',
                    'metadata': {
                        'name': self.object_name('cluster-role'),
                    },
                    'rules': [{
                        'apiGroups': [''],
                        'resources': ['nodes',
                                      'nodes/proxy',
                                      'services',
                                      'endpoints',
                                      'pods'],
                        'verbs': ['get', 'watch', 'list']
                    }]
                }, name=self.BUILDITEM_CLUSTER_ROLE, source=self.SOURCE_NAME, instance=self.basename()),
            ])

        if self.option_get('config.authorization.roles_bind') is not False:
            ret.extend([
                Object({
                    'kind': 'ClusterRoleBinding',
                    'apiVersion': 'rbac.authorization.k8s.io/v1beta1',
                    'metadata': {
                        'name': self.object_name('cluster-role-binding'),
                    },
                    'subjects': [{
                        'kind': 'ServiceAccount',
                        'name': self.object_name('service-account'),
                        'namespace': self.namespace(),
                    }],
                    'roleRef': {
                        'apiGroup': 'rbac.authorization.k8s.io',
                        'kind': 'ClusterRole',
                        'name': self.object_name('cluster-role'),
                    }
                }, name=self.BUILDITEM_CLUSTER_ROLE_BINDING, source=self.SOURCE_NAME, instance=self.basename())
            ])

        return ret

    def internal_build_config(self) -> Sequence[ObjectItem]:
        ret = []

        ret.append(Object({
            'apiVersion': 'v1',
            'kind': 'ConfigMap',
            'metadata': {
                'name': self.object_name('config'),
                'namespace': self.namespace(),
            },
            'data': {
                'promtail.yaml': LiteralStr(self.promtail_configfile_get()),
            }
        }, name=self.BUILDITEM_CONFIG, source=self.SOURCE_NAME, instance=self.basename()))

        return ret

    def internal_build_service(self) -> Sequence[ObjectItem]:
        ret = []

        ret.extend([
            Object({
                'apiVersion': 'apps/v1',
                'kind': 'DaemonSet',
                'metadata': {
                    'name': self.object_name('daemonset'),
                    'namespace': self.namespace(),
                    'labels': {
                        'app': self.object_name('pod-label-app'),
                    },
                },
                'spec': {
                    'selector': {
                        'matchLabels': {
                            'app': self.object_name('pod-label-app'),
                        }
                    },
                    'template': {
                        'metadata': {
                            'labels': {
                                'app': self.object_name('pod-label-app'),
                            },
                            'annotations': ValueData({
                                'prometheus.io/scrape': QuotedStr('true'),
                                'prometheus.io/port': QuotedStr('http-metrics'),
                            }, enabled=self.option_get('config.prometheus_annotation') is not False),
                        },
                        'spec': {
                            'serviceAccountName': self.object_name('service-account'),
                            'containers': [{
                                'name': 'promtail',
                                'image': self.option_get('container.promtail'),
                                'args': [
                                    '-config.file=/etc/promtail/promtail.yaml',
                                    ValueData('-client.url={}/loki/api/v1/push'.format(self.option_get('config.loki_url')),
                                              enabled=self.option_get('config.loki_url') is not None),
                                ],
                                'volumeMounts': [{
                                    'name': 'config',
                                    'mountPath': '/etc/promtail'
                                },
                                {
                                    'name': 'run',
                                    'mountPath': '/run/promtail'
                                },
                                {
                                    'mountPath': '/var/lib/docker/containers',
                                    'name': 'docker',
                                    'readOnly': True
                                },
                                {
                                    'mountPath': '/var/log/pods',
                                    'name': 'pods',
                                    'readOnly': True
                                }],
                                'env': [{
                                    'name': 'HOSTNAME',
                                    'valueFrom': {
                                        'fieldRef': {
                                            'fieldPath': 'spec.nodeName'
                                        }
                                    },
                                }],
                                'ports': [{
                                    'containerPort': 3101,
                                    'name': 'http-metrics'
                                }],
                                'securityContext': {
                                    'readOnlyRootFilesystem': True,
                                    'runAsGroup': 0,
                                    'runAsUser': 0
                                },
                                'readinessProbe': {
                                    'failureThreshold': 5,
                                    'httpGet': {
                                        'path': '/ready',
                                        'port': 'http-metrics'
                                    },
                                    'initialDelaySeconds': 10,
                                    'periodSeconds': 10,
                                    'successThreshold': 1,
                                    'timeoutSeconds': 1
                                },
                                'resources': ValueData(
                                    value=self.option_get('kubernetes.resources.daemonset'),
                                    disabled_if_none=True),
                            }],
                            'tolerations': [{
                                'effect': 'NoSchedule',
                                'key': 'node-role.kubernetes.io/master',
                                'operator': 'Exists'
                            }],
                            'volumes': [{
                                'name': 'config',
                                'configMap': {
                                    'name': self.object_name('config'),
                                    'items': [{
                                        'key': 'promtail.yaml',
                                        'path': 'promtail.yaml',
                                    }]
                                }
                            },
                            {
                                'name': 'run',
                                'hostPath': {
                                    'path': '/run/promtail'
                                }
                            },
                            {
                                'hostPath': {
                                    'path': '/var/lib/docker/containers'
                                },
                                'name': 'docker'
                            },
                            {
                                'hostPath': {
                                    'path': '/var/log/pods'
                                },
                                'name': 'pods'
                            }]
                        }
                    }
                }
            }, name=self.BUILDITEM_DAEMONSET, source=self.SOURCE_NAME, instance=self.basename()),
        ])

        return ret

    def promtail_configfile_get(self) -> str:
        if self.configfile is None:
            configfile = self.option_get('config.promtail_config')
            if configfile is None:
                configfile = PromtailConfigFile()
            if isinstance(configfile, str):
                self.configfile = configfile
            else:
                configfilerender = ConfigFileRenderMulti([
                    ConfigFileRender_Yaml(),
                    ConfigFileRender_RawStr()
                ])
                self.configfile = configfilerender.render(configfile.get_value(self))
        return self.configfile



import unittest

from kubragen import KubraGen
from kubragen.jsonpatch import FilterJSONPatches_Apply, ObjectFilter, FilterJSONPatch
from kubragen.provider import Provider_Generic

from kg_promtail import PromtailBuilder, PromtailOptions


class TestBuilder(unittest.TestCase):
    def setUp(self):
        self.kg = KubraGen(provider=Provider_Generic())

    def test_empty(self):
        promtail_config = PromtailBuilder(kubragen=self.kg)
        self.assertEqual(promtail_config.object_name('config'), 'promtail-config')
        self.assertEqual(promtail_config.object_name('daemonset'), 'promtail')

    def test_basedata(self):
        promtail_config = PromtailBuilder(kubragen=self.kg, options=PromtailOptions({
            'namespace': 'myns',
            'basename': 'mypromtail',
        }))
        self.assertEqual(promtail_config.object_name('config'), 'mypromtail-config')
        self.assertEqual(promtail_config.object_name('daemonset'), 'mypromtail')

        FilterJSONPatches_Apply(items=promtail_config.build(promtail_config.BUILD_SERVICE), jsonpatches=[
            FilterJSONPatch(filters=ObjectFilter(names=[promtail_config.BUILDITEM_DAEMONSET]), patches=[
                {'op': 'check', 'path': '/metadata/name', 'cmp': 'equals', 'value': 'mypromtail'},
                {'op': 'check', 'path': '/metadata/namespace', 'cmp': 'equals', 'value': 'myns'},
            ]),
        ])



from kubragen.configfile import ConfigFileExtension, ConfigFile, ConfigFileExtensionData
from kubragen.options import OptionGetter


class PromtailConfigFileExt_Kubernetes(ConfigFileExtension):
    """
    Promtail configuration extension for Kubernetes scrape config.
    """
    def process(self, configfile: ConfigFile, data: ConfigFileExtensionData, options: OptionGetter) -> None:
        if 'scrape_configs' not in data.data:
            data.data['scrape_configs'] = []
        data.data['scrape_configs'].extend([{
            'job_name': 'kubernetes-pods-name',
            'pipeline_stages': [{'docker': {}}],
            'kubernetes_sd_configs': [{'role': 'pod'}],
            'relabel_configs': [{
                'source_labels': ['__meta_kubernetes_pod_label_name'],
                'target_label': '__service__'
            },
            {
                'source_labels': ['__meta_kubernetes_pod_node_name'],
                'target_label': '__host__'
            },
            {
                'action': 'drop',
                'regex': '',
                'source_labels': ['__service__']
            },
            {
                'action': 'labelmap',
                'regex': '__meta_kubernetes_pod_label_(.+)'
            },
            {
                'action': 'replace',
                'replacement': '$1',
                'separator': '/',
                'source_labels': ['__meta_kubernetes_namespace', '__service__'],
                'target_label': 'job'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_namespace'],
                'target_label': 'namespace'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_pod_name'],
                'target_label': 'pod'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_pod_container_name'],
                'target_label': 'container'
            },
            {
                'replacement': '/var/log/pods/*$1/*.log',
                'separator': '/',
                'source_labels': ['__meta_kubernetes_pod_uid', '__meta_kubernetes_pod_container_name'],
                'target_label': '__path__'
            }]
        },
        {
            'job_name': 'kubernetes-pods-app',
            'pipeline_stages': [{'docker': {}}],
            'kubernetes_sd_configs': [{'role': 'pod'}],
            'relabel_configs': [{
                'action': 'drop',
                'regex': '.+',
                'source_labels': ['__meta_kubernetes_pod_label_name']
            },
            {
                'source_labels': ['__meta_kubernetes_pod_label_app'],
                'target_label': '__service__'
            },
            {
                'source_labels': ['__meta_kubernetes_pod_node_name'],
                'target_label': '__host__'
            },
            {
                'action': 'drop',
                'regex': '',
                'source_labels': ['__service__']
            },
            {
                'action': 'labelmap',
                'regex': '__meta_kubernetes_pod_label_(.+)'
            },
            {
                'action': 'replace',
                'replacement': '$1',
                'separator': '/',
                'source_labels': ['__meta_kubernetes_namespace',
                    '__service__'],
                'target_label': 'job'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_namespace'],
                'target_label': 'namespace'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_pod_name'],
                'target_label': 'pod'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_pod_container_name'],
                'target_label': 'container'
            },
            {
                'replacement': '/var/log/pods/*$1/*.log',
                'separator': '/',
                'source_labels': ['__meta_kubernetes_pod_uid',
                    '__meta_kubernetes_pod_container_name'],
                'target_label': '__path__'
            }]
        },
        {
            'job_name': 'kubernetes-pods-direct-controllers',
            'pipeline_stages': [{'docker': {}}],
            'kubernetes_sd_configs': [{'role': 'pod'}],
            'relabel_configs': [{
                'action': 'drop',
                'regex': '.+',
                'separator': '',
                'source_labels': ['__meta_kubernetes_pod_label_name', '__meta_kubernetes_pod_label_app']
            },
            {
                'action': 'drop',
                'regex': '[0-9a-z-.]+-[0-9a-f]{8,10}',
                'source_labels': ['__meta_kubernetes_pod_controller_name']
            },
            {
                'source_labels': ['__meta_kubernetes_pod_controller_name'],
                'target_label': '__service__'
            },
            {
                'source_labels': ['__meta_kubernetes_pod_node_name'],
                'target_label': '__host__'
            },
            {
                'action': 'drop',
                'regex': '',
                'source_labels': ['__service__']
            },
            {
                'action': 'labelmap',
                'regex': '__meta_kubernetes_pod_label_(.+)'
            },
            {
                'action': 'replace',
                'replacement': '$1',
                'separator': '/',
                'source_labels': ['__meta_kubernetes_namespace',
                    '__service__'],
                'target_label': 'job'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_namespace'],
                'target_label': 'namespace'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_pod_name'],
                'target_label': 'pod'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_pod_container_name'],
                'target_label': 'container'
            },
            {
                'replacement': '/var/log/pods/*$1/*.log',
                'separator': '/',
                'source_labels': ['__meta_kubernetes_pod_uid',
                    '__meta_kubernetes_pod_container_name'],
                'target_label': '__path__'
            }]
        },
        {
            'job_name': 'kubernetes-pods-indirect-controller',
            'pipeline_stages': [{'docker': {}}],
            'kubernetes_sd_configs': [{'role': 'pod'}],
            'relabel_configs': [{
                'action': 'drop',
                'regex': '.+',
                'separator': '',
                'source_labels': ['__meta_kubernetes_pod_label_name',
                    '__meta_kubernetes_pod_label_app']
            },
            {
                'action': 'keep',
                'regex': '[0-9a-z-.]+-[0-9a-f]{8,10}',
                'source_labels': ['__meta_kubernetes_pod_controller_name']
            },
            {
                'action': 'replace',
                'regex': '([0-9a-z-.]+)-[0-9a-f]{8,10}',
                'source_labels': ['__meta_kubernetes_pod_controller_name'],
                'target_label': '__service__'
            },
            {
                'source_labels': ['__meta_kubernetes_pod_node_name'],
                'target_label': '__host__'
            },
            {
                'action': 'drop',
                'regex': '',
                'source_labels': ['__service__']
            },
            {
                'action': 'labelmap',
                'regex': '__meta_kubernetes_pod_label_(.+)'
            },
            {
                'action': 'replace',
                'replacement': '$1',
                'separator': '/',
                'source_labels': ['__meta_kubernetes_namespace',
                    '__service__'],
                'target_label': 'job'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_namespace'],
                'target_label': 'namespace'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_pod_name'],
                'target_label': 'pod'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_pod_container_name'],
                'target_label': 'container'
            },
            {
                'replacement': '/var/log/pods/*$1/*.log',
                'separator': '/',
                'source_labels': ['__meta_kubernetes_pod_uid',
                    '__meta_kubernetes_pod_container_name'],
                'target_label': '__path__'
            }]
        },
        {
            'job_name': 'kubernetes-pods-static',
            'pipeline_stages': [{'docker': {}}],
            'kubernetes_sd_configs': [{'role': 'pod'}],
            'relabel_configs': [{
                'action': 'drop',
                'regex': '',
                'source_labels': ['__meta_kubernetes_pod_annotation_kubernetes_io_config_mirror']
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_pod_label_component'],
                'target_label': '__service__'
            },
            {
                'source_labels': ['__meta_kubernetes_pod_node_name'],
                'target_label': '__host__'
            },
            {
                'action': 'drop',
                'regex': '',
                'source_labels': ['__service__']
            },
            {
                'action': 'labelmap',
                'regex': '__meta_kubernetes_pod_label_(.+)'
            },
            {
                'action': 'replace',
                'replacement': '$1',
                'separator': '/',
                'source_labels': ['__meta_kubernetes_namespace',
                    '__service__'],
                'target_label': 'job'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_namespace'],
                'target_label': 'namespace'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_pod_name'],
                'target_label': 'pod'
            },
            {
                'action': 'replace',
                'source_labels': ['__meta_kubernetes_pod_container_name'],
                'target_label': 'container'
            },
            {
                'replacement': '/var/log/pods/*$1/*.log',
                'separator': '/',
                'source_labels': ['__meta_kubernetes_pod_annotation_kubernetes_io_config_mirror',
                    '__meta_kubernetes_pod_container_name'],
                'target_label': '__path__'
            }],
        }])



from .builder import (
    PromtailBuilder
)
from .option import (
    PromtailOptions
)
from .configfile import (
    PromtailConfigFileOptions,
    PromtailConfigFile,
)
from .configfileext import (
    PromtailConfigFileExt_Kubernetes,
)

__version__ = "0.7.6"

__all__ = [
    'PromtailOptions',
    'PromtailBuilder',
    'PromtailConfigFileOptions',
    'PromtailConfigFile',
    'PromtailConfigFileExt_Kubernetes',
]



from typing import Optional, Any, Sequence, Mapping

from kubragen.configfile import ConfigFileOutput, ConfigFileOutput_Dict, ConfigFile_Extend, \
    ConfigFileExtension, ConfigFileExtensionData
from kubragen.merger import Merger
from kubragen.option import OptionDef
from kubragen.options import Options, option_root_get, OptionGetter


class PromtailConfigFileOptions(Options):
    """
    Options for Promtail config file.

    .. list-table::
        :header-rows: 1

        * - option
          - description
          - allowed types
          - default value
        * - config |rarr| merge_config
          - Mapping to merge into final config
          - Mapping
          -
    """
    def define_options(self) -> Optional[Any]:
        """
        Declare the options for the Promtail config file.

        :return: The supported options
        """
        return {
            'config': {
                'merge_config': OptionDef(allowed_types=[Mapping]),
            },
        }


class PromtailConfigFile(ConfigFile_Extend):
    """
    Promtail main configuration file in Yaml format.
    """
    options: PromtailConfigFileOptions

    def __init__(self, options: Optional[PromtailConfigFileOptions] = None,
                 extensions: Optional[Sequence[ConfigFileExtension]] = None):
        super().__init__(extensions)
        if options is None:
            options = PromtailConfigFileOptions()
        self.options = options

    def option_get(self, name: str):
        return option_root_get(self.options, name)

    def init_value(self, options: OptionGetter) -> ConfigFileExtensionData:
        ret = ConfigFileExtensionData({
            'client': {
                'backoff_config': {
                    'max_period': '5m',
                    'max_retries': 10,
                    'min_period': '500ms'
                },
                'batchsize': 1048576,
                'batchwait': '1s',
                'external_labels': {},
                'timeout': '10s'
            },
            'positions': {
                'filename': '/run/promtail/positions.yaml'
            },
            'server': {
                'http_listen_port': 3101
            },
            'target_config': {
                'sync_period': '10s'
            },
            'scrape_configs': []
        })

        return ret

    def finish_value(self, options: OptionGetter, data: ConfigFileExtensionData) -> ConfigFileOutput:
        if self.option_get('config.merge_config') is not None:
            Merger.merge(data.data, self.option_get('config.merge_config'))
        return ConfigFileOutput_Dict(data.data)



from typing import Optional, Any, Mapping

from kubragen.configfile import ConfigFile
from kubragen.option import OptionDef
from kubragen.options import Options


class PromtailOptions(Options):
    """
    Options for the Promtail builder.

    .. list-table::
        :header-rows: 1

        * - option
          - description
          - allowed types
          - default value
        * - basename
          - object names prefix
          - str
          - ```promtail```
        * - namespace
          - namespace
          - str
          - ```monitoring```
        * - config |rarr| prometheus_annotation
          - add prometheus annotations
          - bool
          - ```False```
        * - config |rarr| promtail_config
          - Promtail config file
          - str, ConfigFile
          - :class:`kg_promtail.PromtailConfigFile`
        * - config |rarr| loki_url
          - Loki url (with http://)
          - str
          -
        * - config |rarr| authorization |rarr| serviceaccount_create
          - whether to create a service account
          - bool
          - ```True```
        * - config |rarr| authorization |rarr| serviceaccount_use
          - service account to use if not creating
          - str
          -
        * - config |rarr| authorization |rarr| roles_create
          - whether create roles
          - bool
          - ```True```
        * - config |rarr| authorization |rarr| roles_bind
          - whether to bind roles to service account
          - bool
          - ```True```
        * - container |rarr| promtail
          - promtail container image
          - str
          - ```grafana/promtail:<version>```
        * - kubernetes |rarr| resources |rarr| daemonset
          - Kubernetes StatefulSet resources
          - dict
          -
    """
    def define_options(self) -> Optional[Any]:
        """
        Declare the options for the Promtail builder.

        :return: The supported options
        """
        return {
            'basename': OptionDef(required=True, default_value='promtail', allowed_types=[str]),
            'namespace': OptionDef(required=True, default_value='monitoring', allowed_types=[str]),
            'config': {
                'prometheus_annotation': OptionDef(required=True, default_value=False, allowed_types=[bool]),
                'promtail_config': OptionDef(allowed_types=[str, ConfigFile]),
                'loki_url': OptionDef(allowed_types=[str]),
                'authorization': {
                    'serviceaccount_create': OptionDef(required=True, default_value=True, allowed_types=[bool]),
                    'serviceaccount_use': OptionDef(allowed_types=[str]),
                    'roles_create': OptionDef(required=True, default_value=True, allowed_types=[bool]),
                    'roles_bind': OptionDef(required=True, default_value=True, allowed_types=[bool]),
                },
            },
            'container': {
                'promtail': OptionDef(required=True, default_value='grafana/promtail:2.0.0', allowed_types=[str]),
            },
            'kubernetes': {
                'resources': {
                    'daemonset': OptionDef(allowed_types=[Mapping]),
                }
            },
        }
